/* convert a structure factor file from one format to another */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "sf_convert.h"
#include "util.h"
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"



void make_sf_standard(char *sffile, char *pdbfile ,char *ofile ,
                      char *correct, char *date,char *date_new, char *pdbid);

void correct_list_file(char *inpf1, char *inpf2)
{
    char  infile[256], ofile[256], id1[20], id2[20], correct[256], command[512];
    char **line1, **line2, date[20],date_new[30], pdbfile[256];
    int  i,j, nline1, nline2;

    strcpy(date_new, "2010-02-16"); //the new date for release

    
    line1=get_lines_from_file(inpf1, &nline1);
    line2=get_lines_from_file(inpf2, &nline2);
    
    printf("Doing sf correction (%d %d)\n", nline1,nline2);
    for(i=0; i<nline1; i++){
        strcpy(date,"?");
        
        sscanf(line1[i],"%s", id1);
        if(strlen(id1)<4 || id1[0] =='!' ) continue;
        
//        sprintf(infile, "r%ssf.ent_new", id1);
        sprintf(infile, "r%ssf.ent.mmcif", id1);//the corrected file
        sprintf(ofile, "r%ssf.ent_new", id1);
        
        sprintf(command, "/bin/csh -c \" auto -pdb  %s \" ",id1);
        system(command);
        sprintf(pdbfile, "pdb%s.ent", id1);
        
        strcpy(correct,line1[i]+5);
        rid_of_front_end_space(correct);

        for(j=0; j<nline2; j++){ //look for date of the ID
            sscanf(line2[j],"%s", id2);
            if(!strcmp_case(id1, id2)){
                sscanf(line2[j],"%*s%*s%s", date);
                break;
            }
        }
        
        make_sf_standard(infile, pdbfile, ofile, correct, date,date_new, id1);
        
    
        printf("\nThe new file is %s\n", ofile);
        
//        sprintf(command, "/bin/csh -c \" rm -f *_tmp? *_temp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
//        system(command);
    }
    if(nline1 && line1) free_memory_2d(line1, nline1);
    if(nline2 && line2) free_memory_2d(line2, nline2);
    

}


void update_audit_new(char *infile,  char *date, char *date_new, char *correction)
{
    char  **line, **aud_date=NULL,**aud_rec=NULL, **aud_id=NULL;
    int id=0, nline, i, j;
    long  naud=0;
    string diags;

    FILE *fout;
    


//    sprintf(outf, "%s_audit", infile);
    printf("Updating audit record(%s)...\n", infile);
    
    
    cifparse(infile, infile, 100) ;
    line=get_lines_from_file(infile, &nline);
    
    
    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(!strncmp_case(line[i], "_audit.", 7)) id=1;
    }

    
    if(id>0){
        CifFile * fobjR = NULL;
        CifParser* cifParserR = NULL;
        fobjR = new CifFile();
        cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
        cifParserR->Parse(infile, diags);
        delete(cifParserR);
        const char *Bid;
        string blockId;

        blockId = fobjR->GetFirstBlockName();
    
        Bid=blockId.c_str();
        aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
        aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);
        aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
        
    }
    fout=fopen(infile, "w");
    
    for(i=0; i<nline; i++){
        if(!strncmp(line[i],"data_", 5)){
            fprintf(fout,"%s", line[i]);
            
            fprintf(fout,"#\nloop_\n");
            fprintf(fout,"_audit.revision_id  \n");
            fprintf(fout,"_audit.creation_date  \n");
            fprintf(fout,"_audit.update_record  \n");

            if(aud_date) {
                for(j=0; j<naud; j++){
                    fprintf(fout,"1_%d  ", j);
                    
                    if (j==0 && aud_date[j][0]=='?'){
                        fprintf(fout," %s ", date );
                    }else{
                        fprintf(fout," %s ", aud_date[j]);
                    }
                    
                    if(j==0 && !aud_rec){
                        fprintf(fout," 'Initial release' \n"); 
                    }else{
                        (aud_rec) ? fprintf(fout," '%s' \n", aud_rec[j]) : fprintf(fout," ? \n");
                    }
                }
                fprintf(fout,"1_%d  %s  '%s' \n", j, date_new, correction);
                
            }else{
                fprintf(fout,"1_0  %s  'Initial release' \n",  date);
                fprintf(fout,"1_1  %s  '%s' \n", date_new, correction);
            }
            
        }else if((strstr(line[i],"loop_") && strstr(line[i+1],"_audit.")) ||
                 strstr(line[i],"_audit.")){//skip audit
            for(j=i; j<nline; j++){
                rid_of_front_space(line[j]);
                if (line[j][0]=='#'){
                    i=j;
                    break;
                }
                continue;
            }
        }else{
            fprintf(fout,"%s", line[i]);
        }
    }
    fclose(fout);

}


void make_sf_standard(char *sffile, char *pdbfile ,char *ofile ,
                      char *correct, char *date, char *date_new,char* pdb_id)
{
    char outf_tmp1[256], outf_tmp2[256], outf_tmp3[256] , **line;
    int i,j, nline, no=0, nf=0, ns=0;
    FILE  *fw,  *fw1, *fw2;
   
    sprintf(outf_tmp1, "%s_temp1", sffile);
    sprintf(outf_tmp2, "%s_temp2", sffile);
    sprintf(outf_tmp3, "%s_temp3", sffile);

    fw=fopen(ofile, "w");
    fw1=fopen(outf_tmp1, "w");
    fw2=fopen(outf_tmp2, "w");
        
    line=get_lines_from_file(sffile, &nline); //seperate 1st and 2st parts
    for(i=0; i<nline; i++){
        if(strstr(line[i],"loop_") && strstr(line[i+1],"_refln.")){
            fprintf(fw2,"data_r%ssf\n", pdb_id);
            for(j=i; j<nline; j++){
                fprintf(fw2,"%s",line[j]);
            }
            i=j;
            
        }else{
            fprintf(fw1,"%s",line[i]);
        }
        
    }
    fclose(fw1);
    fclose(fw2);
    if(line && nline) free_memory_2d(line,nline);
     
    cifparse(outf_tmp1, outf_tmp1, 100) ;
    make_complete_SF(pdbfile, outf_tmp1);
    update_audit_new(outf_tmp1, date, date_new, correct);

    
    line=get_lines_from_file(outf_tmp1, &nline);
    for(i=0; i<nline; i++){
        fprintf(fw, "%s", line[i]);
    }
    if(line && nline) free_memory_2d(line,nline);


    line=get_lines_from_file(outf_tmp2, &nline);
    for(i=0; i<nline; i++){
        if(strstr(line[i], " o")){
            no++;
        }else if(strstr(line[i], " f")){
            nf++;
        }else if (strstr(line[i], "_refln.status")){
            ns=1;
        }else if(i>100 && strstr(line[i], "data_")){
            break;
        }
    }
    if(line && nline) free_memory_2d(line,nline);
    

    if(ns==1 && (no<200 || (nf>0 && no<nf))){ //status exist
        cif2cif_sf_all(outf_tmp2, outf_tmp3, pdb_id, 1);
        line=get_lines_from_file(outf_tmp3, &nline);
    }else{
        cifparse(outf_tmp2, outf_tmp2, 100) ;
        line=get_lines_from_file(outf_tmp2, &nline);
    }
    

    for(i=0; i<nline; i++){
        if(i<10 && strstr(line[i], "data_")) continue;
        fprintf(fw, "%s", line[i]);
    }
    fprintf(fw, "#END OF REFLECTIONS\n");
    if(line && nline) free_memory_2d(line,nline);
    fclose(fw);
}
 

/*            
void insert_column(char *infile, const char *token)
{
    char  **line;
    int i,j,id=0, nline=0;
    FILE *fw=NULL;
    

    line=get_lines_from_file(infile, &nline);

    for(i=0; i<nline; i++){
        if (strstr_case(line[i], token)) {
            id=1;
            printf("Warning! The %s is not in %s and will be inserted as 1\n", token,infile );
            break;
        }
        if(i>400) break;
    }

    if(id <=0) return;

    fw=fopen(infile, "w");
    for(i=0; i<nline; i++){
        fprintf(fw,"%s\n", line[i]);
        
        if(strstr_case(line[i], "loop_") && strstr_case(line[i+1], "_refln.") ){
            fprintf(fw,"%s\n", token);
            fprintf(fw,"%s", line[i+1]);
            for(j=i+2; j<nline; j++){
                if(strchr(line[j], '#')) break;
                if( !strstr_case(line[j], "_refln.")){
                    fprintf(fw, " 1 %s", line[j]);
                }else{
                    fprintf(fw, "%s", line[j]);
                }
            }
        }
    }
    fclose(fw);
    
    if(nline) free_memory_2d(line,nline);
    

}
*/


void add_audit(char *outfile)
{

    char **line;
    int i, nline=0, n=0;
    FILE *fw=NULL;
    
    line=get_lines_from_file(outfile, &nline);
    
    
    for(i=0; i<nline; i++){
        if(strstr_case(line[i], "_audit.")){
            n=1;
            break;
        }
    }

    if(n==0){
        fw=fopen(outfile, "w");
        for(i=0; i<nline; i++){
            if(strstr(line[i],"data_")){
                fprintf(fw,"%s",line[i]);
                audit_sf_header(fw,0);
            }else{
                fprintf(fw,"%s",line[i]);
            }
        }
        fclose(fw);
    }
    if(nline)free_memory_2d(line,nline);
    
}


void update_audit(char *inpf)
// given a list containing pdbid and corrections
{
    char **line, infile[256], ofile[256], id[20], correct[256], command[512];
    int nline=0, i;

    line=get_lines_from_file(inpf, &nline);
    
    for(i=0; i<nline; i++){
        
        strncpy(id,line[i], 4);
        id[4]='\0';
        rid_of_space(id);
        if(strlen(id)<4 || id[0] =='!' ) continue;
        
        sprintf(infile, "r%ssf.ent_new", id);
        sprintf(ofile, "r%ssf.ent_new_audit", id);
        strcpy(correct,line[i]+5);
        rid_of_front_end_space(correct);
        
        add_items_to_cif(infile, ofile, correct);
        
//        reformat_file(ofile, ofile);
    
        printf("\nThe new file is %s\n", ofile);
        
        sprintf(command, "/bin/csh -c \" rm -f *_tmp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
        system(command);
    }

}

void add_items_to_cif(char *inpfile, char *outfile, char *correct)
{
    char  **line, str[512], ftmp1[512], ftmp2[512], cate[80], item[80];
    char **aud_date=NULL,**aud_rec=NULL, date[80] ;
    long  naud=0, na=0;
    int nline, i,j=0,key=0;
    string diags;
    FILE *fout=NULL, *fo1=NULL, *fo2=NULL;


    strcpy(date, "2009-09-01");

//    sprintf(inpfile, "%s_inp_tmp1", inpfile_inp);, inpfile[512]
//    reformat_file(inpfile_inp, inpfile);
    
    line=get_lines_from_file(inpfile, &nline);
    
    sprintf(ftmp1, "%s_tmp1", inpfile);
    sprintf(ftmp2, "%s_tmp2", inpfile);
    
    fout=fopen(outfile,"w");
    fo1=fopen(ftmp1,"w");
    fo2=fopen(ftmp2,"w");

    
    
    for(i=0; i<nline; i++){

        if((strstr(line[i],"loop_") && strstr(line[i+1],"_audit.") && i<nline-2 && key==0) ||
           (strstr(line[i],"_audit.") && key==0) ){
            fprintf(fo1,"data_tmp\n");
            
            for(j=i; j<nline; j++){
                rid_of_front_space(line[j]);
                if(line[j][0]=='#' || (!strncmp(line[j],"loop_", 5)&& j>i) ||
                   (check_category(line[j],cate, item) && !strstr(cate, "audit") )) {
                    fprintf(fo2,"%s", line[j]);
                    i=j+1;
                    
                    break;
                    
                }
                fprintf(fo1,"%s", line[j]);
                na++;
                
            }
            key=1;
            
        }
        
        fprintf(fo2,"%s", line[i]);
    }
    fclose(fo1); fclose(fo2);
    
    if(nline) free_memory_2d(line, nline);
    
    if(na>0){
        CifFile * fobjR = NULL;
        CifParser* cifParserR = NULL;
        fobjR = new CifFile();
        cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
        cifParserR->Parse(ftmp1, diags);
        delete(cifParserR);
        const char *Bid;
        string blockId;

        blockId = fobjR->GetFirstBlockName();
    
        Bid=blockId.c_str();
//        aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
        aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);
        aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
    }
    
    nline=0;
    
    line=get_lines_from_file(ftmp2, &nline);
//    printf("the new file is %s:: %d\n", ftmp2,nline );

    
    for(i=0; i<nline; i++){
        strcpy(str, line[i]);
        rid_of_front_space(str);

        if(!strncmp(str,"data_", 5) && i<200){
            fprintf(fout,"%s", line[i]);
            
//            if(na>0){
                fprintf(fout,"#\nloop_\n");
                fprintf(fout,"_audit.revision_id  \n");
                fprintf(fout,"_audit.creation_date  \n");
                fprintf(fout,"_audit.update_record  \n");
                for(j=0; j<naud; j++){
                     fprintf(fout,"1_%d  ", j);
                    if(aud_date) {
                        rid_of_space(aud_date[j]);
                        fprintf(fout,"%s  ", aud_date[j]) ;
                    }else
                        fprintf(fout," ? ");
                     
                    (aud_rec) ? fprintf(fout,"'%s' ", aud_rec[j]) : fprintf(fout," ? ");
                    fprintf(fout,"\n");
                }
                fprintf(fout,"1_%d  %s  '%s' \n", j, date, correct);
//            }
            
        }else {
            fprintf(fout,"%s", line[i]);
        }
    }
    fclose(fout);
    if(nline)free_memory_2d(line, nline);
    
    
}

